Client-side middleware that can be used with HTTP remotes so that mutations can attach file uploads to mutation parameters.
Client-side middleware that can be used with HTTP remotes so that mutations can attach file uploads to mutation parameters.
Support for converting binary network responses to a usable File URL in a browser. This file is CLJC so the functions exist for SSR, but they do nothing in CLJ.
Support for converting binary network responses to a usable File URL in a browser. This file is CLJC so the functions exist for SSR, but they do nothing in CLJ.
An HTTP driver adapter for http-kit. Suitable for passing as :http-driver to
fulcro-http-remote.
Requires http-kit to be on your classpath.
An HTTP driver adapter for http-kit. Suitable for passing as `:http-driver` to `fulcro-http-remote`. Requires http-kit to be on your classpath.
CLJ HTTP remote for Fulcro. Mirrors the CLJS http-remote middleware API but uses an injected
HTTP driver function instead of Google Closure's XhrIo. This allows headless CLJ apps and tests
to communicate with a real Fulcro server over HTTP.
The :http-driver is a synchronous function with signature:
(fn [{:keys [url method headers body]}]
{:status int, :headers map, :body string, :error nil-or-exception})
IF you have http-kit on your CLASSPATH, you can use the com.fulcrologic.fulcro.networking.http-kit-driver.
The remote runs the driver on a future internally so it does not block the calling thread.
Pass :synchronous? true to fulcro-http-remote to skip the future and run the driver
on the calling thread (required for headless test apps that use synchronous tx processing).
CLJ HTTP remote for Fulcro. Mirrors the CLJS `http-remote` middleware API but uses an injected
HTTP driver function instead of Google Closure's XhrIo. This allows headless CLJ apps and tests
to communicate with a real Fulcro server over HTTP.
The `:http-driver` is a **synchronous** function with signature:
(fn [{:keys [url method headers body]}]
{:status int, :headers map, :body string, :error nil-or-exception})
IF you have http-kit on your CLASSPATH, you can use the `com.fulcrologic.fulcro.networking.http-kit-driver`.
The remote runs the driver on a future internally so it does not block the calling thread.
Pass `:synchronous? true` to `fulcro-http-remote` to skip the future and run the driver
on the calling thread (required for headless test apps that use synchronous tx processing).Simple adapter code that allows you to use a generic parser 'as if' it were a client remote in CLJS.
Simple adapter code that allows you to use a generic parser 'as if' it were a client remote in CLJS.
A wrapper for remotes that adds automatic retry with exponential back-off behavior. This makes the remote more tolerant of flaky network communications, and is a simple wrapper that should work with any remote implementation. NOTE: Fulcro websockets already has automatic retry support which works a bit differently than this one. Be careful that you understand what you are doing if you configure them both at the same time.
See com.fulcrologic.fulcro.offline for features that help support true offline modes of operation.
A wrapper for remotes that adds automatic retry with exponential back-off behavior. This makes the remote more tolerant of flaky network communications, and is a simple wrapper that should work with any remote implementation. NOTE: Fulcro websockets already has automatic retry support which works a bit differently than this one. Be careful that you understand what you are doing if you configure them both at the same time. See `com.fulcrologic.fulcro.offline` for features that help support true offline modes of operation.
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |